package com.shuoeasy.test.dao.impl;

import java.math.BigDecimal;

import org.hibernate.Query;
import org.hibernate.Session;
import org.hibernate.SessionFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;

import com.shuoeasy.test.dao.UserDao;
import com.shuoeasy.test.entity.User;
import com.shuoeasy.test.exceptions.UserException;

@Repository("userDao")
public class UserDaoImpl implements UserDao{
	
	@Autowired
	private SessionFactory sessionFactory;
	
	private Session getSession(){
		return sessionFactory.getCurrentSession();
	}
	
	@Override
	public int add(User user){
		getSession().save(user);
		return 1;
	}
	
	@Override
	public User find(int uid){
		String hql = "FROM User u WHERE u.id = :uid";
		Query query = getSession().createQuery(hql);
		query.setInteger("uid", uid);
		User u =  (User)query.uniqueResult();
		return u;
	}

	/**
	 * 转账
	 */
	@Override
	public int transfer(int fromUid, int toUid, BigDecimal money) {
		// 付款者
		User fromUser = (User) getSession().get(User.class, fromUid);
		if(fromUser == null){
			throw new UserException("付款账户不存在");
		}
		
		// 收款者
		User toUser = (User) getSession().get(User.class, toUid);
		if(toUser == null){
			throw new UserException("收款账户不存在");
		}
		
		BigDecimal fromUserMoney = fromUser.getMoney().subtract(money);
		if(fromUserMoney.compareTo(BigDecimal.ZERO) < 0){
			throw new UserException("付款账户余额不足");
		}
		
		BigDecimal toUserMoney = toUser.getMoney().add(money);
		
		fromUser.setMoney(fromUserMoney);
		toUser.setMoney(toUserMoney);
		
		getSession().update(fromUser);
		getSession().update(toUser);
		
		return 1;
	}

	
	
	 
}
